@@ -1,4 +1,3 @@
-/* $Xorg: smproxy.c,v 1.6 2001/02/09 02:05:35 xorgcvs Exp $ */
 /******************************************************************************
 
 Copyright 1994, 1998  The Open Group
@@ -25,11 +24,15 @@ in this Software without prior written authorization from The Open Group.
 
 Author:  Ralph Mor, X Consortium
 ******************************************************************************/
-/* $XFree86: xc/programs/smproxy/smproxy.c,v 3.8 2001/10/28 03:34:25 tsi Exp $ */
 
 #include "smproxy.h"
 #include <unistd.h>
 #include <X11/Xmu/WinUtil.h>
+#include <limits.h>
+
+#ifndef HOST_NAME_MAX
+#define HOST_NAME_MAX 256
+#endif
 
 static XtAppContext appContext;
 static Display *disp;
@@ -72,7 +75,7 @@ static void SaveCompleteCB ( SmcConn smcConn, SmPointer clientData );
 static void ShutdownCancelledCB ( SmcConn smcConn, SmPointer clientData );
 static void ProcessIceMsgProc ( XtPointer client_data, int *source, XtInputId *id );
 static void NullIceErrorHandler ( IceConn iceConn, Bool swap, 
-			   int offendingMinorOpCode, 
+			   int offendingMinorOpcode,
 			   unsigned long offendingSequence, 
 			   int errorClass, int severity, IcePointer values );
 static void ConnectClientToSM ( WinInfo *winInfo );
@@ -100,7 +103,7 @@ HasSaveYourself(Window window)
 {
     Atom *protocols;
     int numProtocols;
-    int i, found;
+    int found;
 
     protocols = NULL;
 
@@ -111,9 +114,10 @@ HasSaveYourself(Window window)
 
     if (protocols != NULL)
     {
-	for (i = 0; i < numProtocols; i++)
+	for (int i = 0; i < numProtocols; i++) {
 	    if (protocols[i] == wmSaveYourselfAtom)
 		found = 1;
+	}
 
 	XFree (protocols);
     }
@@ -200,7 +204,8 @@ CheckFullyQuantifiedName(char *name, int *newstring)
      * will have a WM_CLIENT_MACHINE that is not fully quantified.
      * For example, we might get "excon" instead of "excon.x.org".
      * This really stinks.  The best we can do is tag on our own
-     * domain name.
+     * domain name, if we can - if not, the unquantified name is
+     * better than nothing.
      */
 
     if (strchr (name, '.') != NULL)
@@ -210,11 +215,13 @@ CheckFullyQuantifiedName(char *name, int *newstring)
     }
     else
     {
-	char hostnamebuf[80];
+	char hostnamebuf[HOST_NAME_MAX + 1] = { 0 };
 	char *firstDot;
 
-	gethostname (hostnamebuf, sizeof hostnamebuf);
-	firstDot = strchr (hostnamebuf, '.');
+	if (gethostname (hostnamebuf, sizeof hostnamebuf) == 0)
+	    firstDot = strchr (hostnamebuf, '.');
+	else
+	    firstDot = NULL;
 
 	if (!firstDot)
 	{
@@ -223,12 +230,12 @@ CheckFullyQuantifiedName(char *name, int *newstring)
 	}
 	else
 	{
-	    int bytes = strlen (name) + strlen (firstDot + 1) + 2;
 	    char *newptr;
 
-	    newptr = (char *) malloc (bytes);
-	    sprintf (newptr, "%s.%s", name, firstDot + 1);
-
+	    if (asprintf (&newptr, "%s.%s", name, firstDot + 1) == -1) {
+                *newstring = 0;
+                return (name);
+            }
 	    *newstring = 1;
 	    return (newptr);
 	}
@@ -242,7 +249,6 @@ FinishSaveYourself(WinInfo *winInfo, Bool has_WM_SAVEYOURSELF)
 {
     SmProp prop1, prop2, prop3, *props[3];
     SmPropValue prop1val, prop2val, prop3val;
-    int i;
 
     if (!winInfo->got_first_save_yourself)
     {
@@ -257,7 +263,7 @@ FinishSaveYourself(WinInfo *winInfo, Bool has_WM_SAVEYOURSELF)
 	prop1val.value = (SmPointer) winInfo->wm_command[0];
 	prop1val.length = strlen (winInfo->wm_command[0]);
     
-	sprintf (userId, "%ld", (long)getuid());
+	snprintf (userId, sizeof(userId), "%ld", (long)getuid());
 	prop2.name = SmUserID;
 	prop2.type = SmARRAY8;
 	prop2.num_vals = 1;
@@ -267,7 +273,8 @@ FinishSaveYourself(WinInfo *winInfo, Bool has_WM_SAVEYOURSELF)
     
 	fullyQuantifiedName = CheckFullyQuantifiedName (
 	    (char *) winInfo->wm_client_machine.value, &newstring);
-	sprintf (restartService, "rstart-rsh/%s", fullyQuantifiedName);
+	snprintf (restartService, sizeof(restartService),
+                  "rstart-rsh/%s", fullyQuantifiedName);
 	if (newstring)
 	    free (fullyQuantifiedName);
 
@@ -291,8 +298,7 @@ FinishSaveYourself(WinInfo *winInfo, Bool has_WM_SAVEYOURSELF)
     prop1.type = SmLISTofARRAY8;
     prop1.num_vals = winInfo->wm_command_count;
     
-    prop1.vals = (SmPropValue *) malloc (
-	winInfo->wm_command_count * sizeof (SmPropValue));
+    prop1.vals = calloc (winInfo->wm_command_count, sizeof (SmPropValue));
     
     if (!prop1.vals)
     {
@@ -300,7 +306,7 @@ FinishSaveYourself(WinInfo *winInfo, Bool has_WM_SAVEYOURSELF)
 	return;
     }
 
-    for (i = 0; i < winInfo->wm_command_count; i++)
+    for (int i = 0; i < winInfo->wm_command_count; i++)
     {
 	prop1.vals[i].value = (SmPointer) winInfo->wm_command[i];
 	prop1.vals[i].length = strlen (winInfo->wm_command[i]);
@@ -316,7 +322,7 @@ FinishSaveYourself(WinInfo *winInfo, Bool has_WM_SAVEYOURSELF)
     
     SmcSetProperties (winInfo->smc_conn, 2, props);
     
-    free ((char *) prop1.vals);
+    free (prop1.vals);
     
     /*
      * If the client doesn't support WM_SAVE_YOURSELF, we should
@@ -575,7 +581,7 @@ AddNewWindow(Window window)
     if (LookupWindow (window, NULL, NULL))
 	return (NULL);
 
-    newptr = (WinInfo *) malloc (sizeof (WinInfo));
+    newptr = malloc (sizeof (WinInfo));
 
     if (newptr == NULL)
 	return (NULL);
@@ -884,8 +890,8 @@ ProxySaveYourselfPhase2CB(SmcConn smcConn, SmPointer clientData)
     Bool success = True;
     SmProp prop1, prop2, prop3, *props[3];
     SmPropValue prop1val, prop2val, prop3val;
-    char discardCommand[80];
-    int numVals, i;
+    char *discardCommand;
+    int numVals;
     static int first_time = 1;
 
     if (first_time)
@@ -900,7 +906,7 @@ ProxySaveYourselfPhase2CB(SmcConn smcConn, SmPointer clientData)
 	prop1val.value = Argv[0];
 	prop1val.length = strlen (Argv[0]);
 
-	sprintf (userId, "%ld", (long)getuid());
+	snprintf (userId, sizeof(userId), "%ld", (long)getuid());
 	prop2.name = SmUserID;
 	prop2.type = SmARRAY8;
 	prop2.num_vals = 1;
@@ -933,8 +939,7 @@ ProxySaveYourselfPhase2CB(SmcConn smcConn, SmPointer clientData)
     prop1.name = SmRestartCommand;
     prop1.type = SmLISTofARRAY8;
 
-    prop1.vals = (SmPropValue *) malloc (
-	(Argc + 4) * sizeof (SmPropValue));
+    prop1.vals = calloc ((Argc + 4), sizeof (SmPropValue));
 
     if (!prop1.vals)
     {
@@ -944,7 +949,7 @@ ProxySaveYourselfPhase2CB(SmcConn smcConn, SmPointer clientData)
 
     numVals = 0;
 
-    for (i = 0; i < Argc; i++)
+    for (int i = 0; i < Argc; i++)
     {
 	if (strcmp (Argv[i], "-clientId") == 0 ||
 	    strcmp (Argv[i], "-restore") == 0)
@@ -973,7 +978,10 @@ ProxySaveYourselfPhase2CB(SmcConn smcConn, SmPointer clientData)
     prop1.num_vals = numVals;
 
 
-    sprintf (discardCommand, "rm %s", filename);
+    if (asprintf (&discardCommand, "rm %s", filename) == -1) {
+	success = False;
+	goto finishUp;
+    }
     prop2.name = SmDiscardCommand;
     prop2.type = SmARRAY8;
     prop2.num_vals = 1;
@@ -985,7 +993,8 @@ ProxySaveYourselfPhase2CB(SmcConn smcConn, SmPointer clientData)
     props[1] = &prop2;
 
     SmcSetProperties (smcConn, 2, props);
-    free ((char *) prop1.vals);
+    free (prop1.vals);
+    free (discardCommand);
 
  finishUp:
 
@@ -1108,9 +1117,8 @@ ConnectProxyToSM(char *previous_id)
 static void
 CheckForExistingWindows(Window root)
 {
-    Window dontCare1, dontCare2, *children, client_window;
+    Window dontCare1, dontCare2, *children;
     unsigned int nchildren, i;
-    XCreateWindowEvent event;
 
     /*
      * We query the root tree for all windows created thus far.
@@ -1123,7 +1131,8 @@ CheckForExistingWindows(Window root)
 
     for (i = 0; i < nchildren; i++)
     {
-	event.window = children[i];
+	Window client_window;
+	XCreateWindowEvent event = { .window = children[i] };
 
 	HandleCreate (&event);
 
@@ -1150,6 +1159,8 @@ main (int argc, char *argv[])
     char *restore_filename = NULL;
     char *client_id = NULL;
     int i, zero = 0;
+    const char * const usage =
+        "usage:  %s [-clientId id] [-restore file] [-debug] [-version]\n";
 
     Argc = argc;
     Argv = argv;
@@ -1165,21 +1176,45 @@ main (int argc, char *argv[])
 		continue;
 
 	      case 'c':				/* -clientId */
-		if (++i >= argc) goto usage;
+		if (++i >= argc) {
+		    fprintf (stderr, "%s: -clientId requires an argument\n",
+			     argv[0]);
+		    goto usage;
+		}
 		client_id = argv[i];
 		continue;
 
 	      case 'r':				/* -restore */
-		if (++i >= argc) goto usage;
+		if (++i >= argc) {
+		    fprintf (stderr, "%s: -restore requires an argument\n",
+			     argv[0]);
+		    goto usage;
+		}
 		restore_filename = argv[i];
 		continue;
+
+	      case 'v':
+		puts (PACKAGE_STRING);
+		exit (0);
+
+            case '-':
+                if (strcmp(argv[i], "--help") == 0) {
+                    fprintf (stdout, usage, argv[0]);
+                    exit (0);
+                }
+                if (strcmp(argv[i], "--version") == 0) {
+                    puts (PACKAGE_STRING);
+                    exit (0);
+                }
+                break;
 	    }
 	}
 
+	fprintf (stderr, "%s: unrecognized argument: %s\n", argv[0], argv[i]);
+
     usage:
 
-	fprintf (stderr,
-	    "usage:  %s [-clientId id] [-restore file] [-debug]\n", argv[0]);
+	fprintf (stderr, usage, argv[0]);
 	exit (1);
     }
 
